;;; DO NOT MODIFY THIS FILE
(if (featurep 'mew-autoloads) (error "Already loaded"))

;;;### (autoloads nil "_pkg" "mew/_pkg.el")

(package-provide 'mew :version 1.18 :author-version "1.94.2" :type 'regular)

;;;***

;;;### (autoloads (mew-user-agent-compose mew-send mew) "mew" "mew/mew.el")

(autoload 'mew "mew" "\
Execute Mew. If 'mew-auto-get' is 't', messages stored in your
spool are fetched to the +inbox folder and messages in the +inbox
folder are listed up in Summary mode. If 'mew-auto-get' is 'nil', list
up messages in the inbox folder. If '\\[universal-argument]' is specified, perform this
function thinking that 'mew-auto-get' is reversed." t nil)

(autoload 'mew-send "mew" "\
Execute Mew then prepare a draft. This may be used as library
function." t nil)

(autoload 'mew-user-agent-compose "mew" "\
Set up mail composition draft with Mew.
This is 'mail-user-agent' entry point to Mew.

The optional arguments TO and SUBJECT specify recipients and the
initial Subject field, respectively.

OTHER-HEADERS is an alist specifying additional
header fields.  Elements look like (HEADER . VALUE) where both
HEADER and VALUE are strings.

A Draft buffer is prepared according to SWITCH-FUNCTION.

CONTINUE, YANK-ACTION and SEND-ACTIONS are ignored." nil nil)

;;;***

(provide 'mew-autoloads)
